package io.pivotal.security.request;

import static com.greghaskins.spectrum.Spectrum.describe;
import static com.greghaskins.spectrum.Spectrum.it;
import static io.pivotal.security.helper.JsonTestHelper.deserialize;
import static io.pivotal.security.helper.JsonTestHelper.deserializeAndValidate;
import static io.pivotal.security.helper.JsonTestHelper.hasViolationWithMessage;
import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.contains;
import static org.hamcrest.Matchers.equalTo;
import static org.hamcrest.Matchers.instanceOf;

import com.greghaskins.spectrum.Spectrum;
import java.util.Set;
import javax.validation.ConstraintViolation;
import org.junit.runner.RunWith;

@RunWith(Spectrum.class)
public class PasswordSetRequestTest {

  {
    it("should deserialize to PasswordSetRequest", () -> {
      String json = "{"
          + "\"name\":\"some-name\","
          + "\"type\":\"password\","
          + "\"value\":\"fake-password\","
          + "\"overwrite\":true"
          + "}";
      BaseCredentialSetRequest deserialize = deserialize(json, BaseCredentialSetRequest.class);

      assertThat(deserialize, instanceOf(PasswordSetRequest.class));
    });

    it("should be valid if all fields are set", () -> {
      String json = "{"
          + "\"name\":\"some-name\","
          + "\"type\":\"password\","
          + "\"value\":\"fake-password\","
          + "\"overwrite\":true"
          + "}";
      Set<ConstraintViolation<BaseCredentialSetRequest>> constraintViolations =
          deserializeAndValidate(json, BaseCredentialSetRequest.class);

      assertThat(constraintViolations.size(), equalTo(0));
    });

    describe("when password is not set", () -> {
      it("should be invalid", () -> {
        String json = "{"
            + "\"name\":\"some-name\","
            + "\"type\":\"password\","
            + "\"overwrite\":true"
            + "}";
        Set<ConstraintViolation<BaseCredentialSetRequest>> constraintViolations =
            deserializeAndValidate(json, BaseCredentialSetRequest.class);

        assertThat(constraintViolations, contains(hasViolationWithMessage("error.missing_value")));
      });
    });

    describe("when password is empty", () -> {
      it("should be invalid", () -> {
        String json = "{"
            + "\"name\":\"some-name\","
            + "\"type\":\"password\","
            + "\"overwrite\":true,"
            + "\"value\":\"\""
            + "}";
        Set<ConstraintViolation<BaseCredentialSetRequest>> constraintViolations =
            deserializeAndValidate(json, BaseCredentialSetRequest.class);

        assertThat(constraintViolations, contains(hasViolationWithMessage("error.missing_value")));
      });
    });
  }
}
